@@ -65,7 +65,7 @@ impl<'a> RequestBuilder<'a> {
     }
 
     ///Replaces all it's headers with headers passed to the function
-    pub fn headers(&mut self, headers: HashMap<String, String>) -> &mut Self {
+    pub fn set_headers(&mut self, headers: HashMap<String, String>) -> &mut Self {
         self.headers = headers;
         self
     }
@@ -89,22 +89,62 @@ impl<'a> RequestBuilder<'a> {
     ///Sends HTTP request. Opens TCP connection, writes request message to stream.
     ///Returns response if operation suceeded.
     pub fn send(&self) -> Result<Response, Box<Error>> {
+        let res;
         let msg = self.parse_msg();
-        let mut res = Vec::new();
         let mut stream = self.connect()?;
 
         if self.url.scheme() == "https" {
             let mut stream = self.secure_conn(stream)?;
-            stream.write_all(&msg)?;
-            stream.flush()?;
-            stream.read_to_end(&mut res)?;
+            res = self.handle_stream(&mut stream, &msg)?;
         } else {
-            stream.write_all(&msg)?;
-            stream.flush()?;
-            stream.read_to_end(&mut res)?;
+            res = self.handle_stream(&mut stream, &msg)?;
         }
 
-        Response::try_from(&res)
+        Ok(res)
+    }
+
+    //Writes message to stream. Reads server response.
+    fn handle_stream<T>(&self, stream: &mut T, msg: &[u8]) -> Result<Response, Box<Error>>
+    where
+        T: Write + Read,
+    {
+        stream.write_all(msg)?;
+        stream.flush()?;
+
+        let (res_head, body_start) = Self::res_head(stream)?;
+        let mut res = Response::new(&res_head)?;
+
+        if self.method != Method::HEAD {
+            let mut res_body = Vec::with_capacity(res.content_len()?);
+            stream.read_to_end(&mut res_body)?;
+            res.append_body(body_start);
+            res.append_body(res_body);
+        }
+
+        Ok(res)
+    }
+
+    fn res_head<T>(stream: &mut T) -> Result<(Vec<u8>, Vec<u8>), Box<Error>>
+    where
+        T: Write + Read,
+    {
+        let mut res_head = Vec::new();
+        let pos;
+
+        loop {
+            let mut buf = [0; 200];
+            let size = stream.read(&mut buf)?;
+            res_head.append(&mut buf[..size].to_vec());
+
+            if let Some(v) = response::find_slice(&res_head, &response::CR_LF_2) {
+                pos = v;
+                break;
+            }
+        }
+
+        let (res_head, res_body) = res_head.split_at(pos);
+
+        Ok((res_head.to_vec(), res_body.to_vec()))
     }
 
     ///Parses request message
@@ -133,7 +173,7 @@ impl<'a> RequestBuilder<'a> {
         request_msg
     }
 
-    //Creates default headers for a request
+    //Creates default headers for a `Request`
     fn default_headers<T: ToString>(host: T) -> HashMap<String, String> {
         let mut headers = HashMap::with_capacity(4);
         headers.insert("Host".to_string(), host.to_string());
@@ -173,9 +213,10 @@ mod tests {
 
     const URL: &str = "http://doc.rust-lang.org/std/string/index.html";
     const URL_S: &str = "https://doc.rust-lang.org/std/string/index.html";
+    const BODY: [u8; 14] = [78, 97, 109, 101, 61, 74, 97, 109, 101, 115, 43, 74, 97, 121];
 
     #[test]
-    fn new_request_b() {
+    fn request_b_new() {
         RequestBuilder::new(URL.parse().unwrap());
         RequestBuilder::new(URL_S.parse().unwrap());
     }
@@ -188,7 +229,7 @@ mod tests {
     }
 
     #[test]
-    fn request_b_headers() {
+    fn request_b_set_headers() {
         let mut req = RequestBuilder::new(URL.parse().unwrap());
         let mut headers = HashMap::with_capacity(4);
         headers.insert("Accept-Charset".to_string(), "utf-8".to_string());
@@ -196,7 +237,7 @@ mod tests {
         headers.insert("Host".to_string(), "doc.rust-lang.org".to_string());
         headers.insert("Connection".to_string(), "Close".to_string());
 
-        let req = req.headers(headers.clone());
+        let req = req.set_headers(headers.clone());
 
         assert_eq!(req.headers, headers);
     }
@@ -217,11 +258,31 @@ mod tests {
         assert_eq!(req.headers, expect_headers);
     }
 
+    #[test]
+    fn request_b_body() {
+        let mut req = RequestBuilder::new(URL.parse().unwrap());
+        let req = req.body(&BODY);
+
+        assert_eq!(req.body, Some(BODY.as_ref()));
+    }
+
     #[ignore]
     #[test]
-    fn request_b_connect() {
+    fn request_b_send() {
+        RequestBuilder::new(URL.parse().unwrap()).send().unwrap();
+        RequestBuilder::new(URL_S.parse().unwrap()).send().unwrap();
+    }
+
+    #[ignore]
+    #[test]
+    fn request_b_handle_stream() {
         let req = RequestBuilder::new(URL.parse().unwrap());
-        req.connect().unwrap();
+        let mut stream = req.connect().unwrap();
+        let msg = req.parse_msg();
+
+        let res = req.handle_stream(&mut stream, &msg).unwrap();
+
+        assert_ne!(res.status_code(), 400);
     }
 
     #[test]
@@ -242,9 +303,18 @@ mod tests {
 
     #[ignore]
     #[test]
-    fn send_request_b() {
-        RequestBuilder::new(URL.parse().unwrap()).send().unwrap();
-        RequestBuilder::new(URL_S.parse().unwrap()).send().unwrap();
+    fn request_b_connect() {
+        let req = RequestBuilder::new(URL.parse().unwrap());
+        req.connect().unwrap();
+    }
+
+    #[ignore]
+    #[test]
+    fn request_b_secure_conn() {
+        let req = RequestBuilder::new(URL.parse().unwrap());
+        let stream = req.connect().unwrap();
+
+        req.secure_conn(stream).unwrap();
     }
 
     #[ignore]
